//
// Authors:
//   Patrick Cernko <cernko@ps.uni-sb.de>
//
// Copyright:
//   Patrick Cernko, 2004
//
// Last Change:
//   $Date: 2004/09/06 13:00:57 $ by $Author: errror $
//   $Revision: 1.1.2.1 $
//
#ifndef __JAVA__DEFAULT_KAFFE_ACCESSORS_HH__
#define __JAVA__DEFAULT_KAFFE_ACCESSORS_HH__

#if defined(INTERFACE)
#pragma interface "java/DefaultKaffeAccessors.hh"
#endif

/*
 * This struct serves to keep information about an exception
 * that will be thrown later if some operation failed.
 */
typedef struct _errorInfo {
        int type;               /* type of error */
        const char* classname;  /* full dotted name of exception/error class */
        char* mess;             /* accompanying msg */
//         struct Hjava_lang_Throwable* throwable; /* a throwable */
} errorInfo;


#define	CDBG(s) 	DBG(INT_VMCALL, s)
#define	RDBG(s) 	DBG(INT_RETURN, s)
#define	NDBG(s) 	DBG(INT_NATIVE, s)
// this gives us redundant debug output!
#define	IDBG(s)		/*DBG(INT_INSTR, s) */
#define	CHDBG(s)	DBG(INT_CHECKS, s)


#define	define_insn(code)                          \
  break;					   \
  case Instr::code:				   \
  IDBG( dprintf("%03ld: %s\n", (long) pc, #code); )

#define	define_insn_alias(code)	                   \
  case Instr::code:				   \
  IDBG( dprintf("%03ld: %s\n", (long) pc, #code); )

#define	define_wide_insn(code)                     \
  break;					   \
  case Instr::code:				   \
  IDBG( dprintf("%03ld: %s\n", (long) pc, #code); )


#define EXPLICIT_CHECK_NULL(_i, _s, _n)                   \
  cbranch_ref_const_ne((_s), 0, reference_label(_i, _n)); \
  softcall_nullpointer();                                 \
  set_label(_i, _n)

/* Define CREATE_NULLPOINTER_CHECKS in md.h when your machine cannot use the
 * MMU for detecting null pointer accesses */
// XXX: according to kaffe's md.h templates, CREATE_NULLPOINTER_CHECKS
//      is _defined_ for win32 but _not_defined_ for i386-linux
#if defined(CREATE_NULLPOINTER_CHECKS)
#define CHECK_NULL(_i, _s, _n)  EXPLICIT_CHECK_NULL(_i, _s, _n)
#else
#define CHECK_NULL(_i, _s, _n)
#endif

/* For JIT3 compatibility */
/* XXX: todo */
#define check_array_store(a,b)		/*softcall_checkarraystore(a,b)*/
#define explicit_check_null(x,obj,y)	EXPLICIT_CHECK_NULL(x,obj,y)
#define check_null(x,obj,y)		CHECK_NULL(x,obj,y)
#define check_div(x,obj,y)
#define check_div_long(x,obj,y)


#define getopcode()             ((int)code[pc])
#define getpc(n)		(code[pc+1+(n)])
#define getcode(n)		(code[(n)])


/* -------------------------------------------------------------------- */
/* Methods */
/* -------------------------------------------------------------------- */
#define	get_method_info_noerror(idx)           get_method_info(idx)
#define	get_static_method_info_noerror(idx)    get_static_method_info(idx)
#define get_special_method_info_noerror(idx)   get_special_method_info(idx)
#define	get_interface_method_info_noerror(idx) get_interface_method_info(idx)

#define	method_name()		(cinfo.name)
#define	method_sig()		(cinfo.signature)
#define method_idx()		(cinfo.method->idx)
#define method_method()		(cinfo.method)
#define method_class()		(cinfo.cls)
#define method_classname()	(cinfo.cname)
#define	method_nargs()		(cinfo.in)
#define	method_returntype()	(cinfo.rettype)


/* -------------------------------------------------------------------- */
/* Fields */
/* -------------------------------------------------------------------- */
#define field_class()		(finfo.cls)
#define field_field()		(finfo.field)
#define field_name()		(finfo.name)
#define field_classname()	(finfo.cname)
#define field_sig()		(finfo.signature)
#define field_statics()		(finfo.cls->static_data)


/* -------------------------------------------------------------------- */
/* Classes */
/* -------------------------------------------------------------------- */
#define	get_class_info_noerror(_idx) get_class_info(_idx)

#define	class_object()		(crinfo)

#define compile_time_error(EINFO) throwError(&EINFO)

#define	SOFT_ADDREFERENCE(_f, _t)
#define	SOFT_ADDREFERENCE_STATIC(_f, _t)

#define	switchpair_size		8
#define	switchpair_addr		4
#define	switchtable_shift	2

#define METHOD_TRANSLATED(M)		((M)->accflags & ACC_TRANSLATED)
#define	METHOD_NATIVECODE(M)		((M)->ncode)

#define class_uninitialized(c) ((c)->state != CSTATE_COMPLETE)
#define class_is_interface(c) ((c)->accflags & ACC_INTERFACE)
#define field_is_static(f) ((f)->accflags & ACC_STATIC)
#define field_is_final(f) ((f)->accflags & ACC_FINAL)
#define method_from_class(m,c) ((m)->cls == (c))

#define method_is_static(m) ((m)->accflags & ACC_STATIC)
#define method_is_constructor(m) ((m)->accflags & ACC_CONSTRUCTOR)
#define method_is_virtual(m) ((m)->idx >= 0)

enum all_access_flags {
    ACC_PUBLIC       = 0x0001,
    ACC_PRIVATE      = 0x0002,
    ACC_PROTECTED    = 0x0004,
    ACC_STATIC       = 0x0008,
    ACC_FINAL        = 0x0010,
    ACC_SYNCHRONIZED = 0x0020,
    ACC_SUPER        = 0x0020,
    ACC_VOLATILE     = 0x0040,
    ACC_TRANSIENT    = 0x0080,
    ACC_NATIVE       = 0x0100,
    ACC_INTERFACE    = 0x0200,
    ACC_ABSTRACT     = 0x0400,
    ACC_STRICT       = 0x0800,
    // special flags for kaffe
    ACC_CONSTRUCTOR  = 0x0800,
    ACC_TRANSLATED   = 0x4000
};


/*
 * slots.h
 *
 * Copyright (c) 1996, 1997
 *	Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */

#define	push(N)			sp += (N)
#define	pop(N)			sp -= (N)

#define	stack(N)		(&sp[-(N)])
#define	stack_long(N)		(&sp[-(N)-1])
#define	stack_ref		stack
#define	stack_float		stack
#define	stack_double		stack_long

#define	rstack			stack
#define	wstack			stack
#define	rstack_ref		stack_ref
#define	wstack_ref		stack_ref
#define	rstack_long		stack_long
#define	wstack_long		stack_long
#define	rstack_float		stack_float
#define	wstack_float		stack_float
#define	rstack_double		stack_double
#define	wstack_double		stack_double

#define	local(N)		(&lcl[(N)])
#define local_ref		local
#define local_long		local
#define local_float		local
#define local_double		local_long

#define	slot_alloctmp(s)	/* Does nothing for interpreter */
#define	slot_freetmp(s)		/* Does nothing for interpreter */
#define	slot_nowriteback(s)	/* Does nothing for interpreter */
#define	slot_nowriteback2(s)	/* Does nothing for interpreter */

/* NB. This should match jvalue and eventually be replaced by it */
typedef struct _slots {
        union {
                jint            tint;
                jword           tword;
                jlong           tlong;
                jfloat          tfloat;
                jdouble         tdouble;
                void*           taddr;
                char*           tstr;
        } v;
} slots;

#endif
