//
// Authors:
//   Patrick Cernko <cernko@ps.uni-sb.de>
//
// Copyright:
//   Patrick Cernko, 2004
//
// Last Change:
//   $Date: 2004/09/06 13:00:57 $ by $Author: errror $
//   $Revision: 1.1.2.2 $
//

#if defined(INTERFACE)
#pragma implementation "java/KaffeFrame.hh"
#endif

#include "KaffeFrame.hh"

// void KaffeFrame::SetTop(u_int top) {
//   StackFrame::ReplaceArg(TOP_POS, top);
// }
// u_int KaffeFrame::GetTop() {
//   return Store::DirectWordToInt(StackFrame::GetArg(TOP_POS));
// }

// word* KaffeFrame::GetBase() {
//   return STATIC_CAST(word*, this) - StackFrame::BASE_SIZE;
// }

// KaffeFrame Accessors
// u_int KaffeFrame::GetSize() {
//   return Store::DirectWordToInt(StackFrame::GetArg(SIZE_POS));
// }

// s_int KaffeFrame::GetPC() {
//   return Store::DirectWordToInt(StackFrame::GetArg(PC_POS));
// }

// void KaffeFrame::SetPC(s_int pc) {
//   StackFrame::InitArg(PC_POS, pc);
// }

// s_int KaffeFrame::GetContPC() {
//   return Store::DirectWordToInt(StackFrame::GetArg(CONT_PC_POS));
// }

// void KaffeFrame::SetContPC(s_int pc) {
//   StackFrame::InitArg(CONT_PC_POS, pc);
// }

// Chunk* KaffeFrame::GetCode() {
//   return Store::DirectWordToChunk(StackFrame::GetArg(CODE_POS));
// }

// RuntimeConstantPool* KaffeFrame::GetRuntimeConstantPool() {
//   return RuntimeConstantPool::FromWordDirect(StackFrame::GetArg(POOL_POS));
// }

// Table* KaffeFrame::GetExceptionTable() {
//   JavaByteCode *byteCode =
//     JavaByteCode::FromWord(StackFrame::GetArg(BYTE_CODE_POS));
//   return byteCode->GetExceptionTable();
// }

// MethodInfo* KaffeFrame::GetMethodInfo() {
//   JavaByteCode *byteCode =
//     JavaByteCode::FromWord(StackFrame::GetArg(BYTE_CODE_POS));
//   return byteCode->GetMethodInfo();
// }

// #define MAX_LOCALS (JavaByteCode::FromWord(StackFrame::GetArg(BYTE_CODE_POS))->GetMaxLocals())
// #define STACK_ZERO (BASE_SIZE + MAX_LOCALS)
// #define STACK_MAX (STACK_ZERO + JavaByteCode::FromWord(StackFrame::GetArg(BYTE_CODE_POS))->GetMaxStack())

// Environment Accessors
// word* KaffeFrame::GetLcl(u_int n) {
//   Assert(n < MAX_LOCALS);
//   // StackFrames store their members upside down
//   return GetBase() - (BASE_SIZE + n);
// }

// Stack Accessors
// void KaffeFrame::push(u_int n) {
//   u_int curTop = GetTop();
// #ifdef DEBUG_LEVEL5
//   fprintf(stderr, "KaffeFrame::push(%d): new Top: %d\n",n,curTop+n);
// #endif
//   Assert(curTop + n - 1 <= STACK_MAX);
//   SetTop(curTop + n);
// }

// void KaffeFrame::pop(u_int n) {
//   u_int curTop = GetTop();
// #ifdef DEBUG_LEVEL5
//   fprintf(stderr, "KaffeFrame::pop(%d): new Top: %d\n",n,curTop-n);
// #endif
//   Assert(curTop - n >= STACK_ZERO);
//   SetTop(curTop - n);
// }

// word* KaffeFrame::GetStack(int n) {
// #ifdef DEBUG_LEVEL5
//   fprintf(stderr,"KaffeFrame::GetStack(%d)",n);
// #endif
//   Assert(n <= 0);
//   u_int curTop = GetTop();
// #ifdef DEBUG_LEVEL5
//   fprintf(stderr," Top: %d, this: %p, Base: %p, Pos: %p\n",
//           curTop,
//           STATIC_CAST(word*, this),
//           GetBase(),
//           (GetBase() - (curTop + n)));
// #endif
//   Assert(curTop + n >= STACK_ZERO);
//   // StackFrames store their members upside down
//   return GetBase() - (curTop + n - 1);
// }

// word KaffeFrame::GetStackValue(int n) {
//   return *GetStack(n);
// }

#ifdef DEBUG_KAFFEFRAMEID
u_int KaffeFrame::GetUniqueID() {
  return Store::DirectWordToInt(StackFrame::GetArg(UNIQUEID_POS));
}
#endif

// KaffeFrame Constructor
KaffeFrame* KaffeFrame::New(Interpreter *interpreter,
                            s_int pc,
                            s_int contPC,
                            JavaByteCode *byteCode,
                            word runtimeConstantPool) {

#ifdef JAVA_INTERPRETER_DEBUG
  fprintf(stderr, "KaffeFrame::New(%p, %d, %d, %p, 0x%x) [%s::%s] == ",
          interpreter,
          pc,
          contPC,
          byteCode,
#ifdef DEBUG_NOPOINTERS
          runtimeConstantPool,
#else
          0,
#endif
          byteCode->GetMethodInfo()->GetClassName()->ExportC(),
          byteCode->GetMethodInfo()->GetName()->ExportC());
#endif

  u_int maxLocals   = byteCode->GetMaxLocals();
  u_int maxStack    = byteCode->GetMaxStack();
  u_int frSize      = BASE_SIZE + maxLocals + maxStack;
  NEW_STACK_FRAME(frame, interpreter, frSize);

#ifdef JAVA_INTERPRETER_DEBUG
    fprintf(stderr, "%p\n", frame);
#endif

  frame->InitArg(SIZE_POS, frame->GetSize() + frSize);
  frame->InitArg(PC_POS, pc);
  frame->InitArg(CONT_PC_POS, contPC);
  frame->InitArg(TOP_POS, BASE_SIZE + maxLocals);
  frame->InitArg(CODE_POS, byteCode->GetCode()->ToWord());
  frame->InitArg(POOL_POS, runtimeConstantPool);
  frame->InitArg(BYTE_CODE_POS, byteCode->ToWord());

#ifdef DEBUG_KAFFEFRAMEID
  static u_int uniqueIdCounter = 0;
  frame->InitArg(UNIQUEID_POS, Store::IntToWord(uniqueIdCounter++));
  fprintf(stderr, "Frame %d: UNIQUEID is stored at pos %d. %d Locals, %d Stack entries.\n", 
          (uniqueIdCounter-1),
          (StackFrame::BASE_SIZE + UNIQUEID_POS),
          maxLocals, maxStack);
#endif

  return STATIC_CAST(KaffeFrame *, frame);
}

#ifdef DEBUG_CHECK
void KaffeFrame::Dump() {
#ifdef DEBUG_KAFFEFRAMEID
  word* wuid = STATIC_CAST(word*, this) - (StackFrame::BASE_SIZE + UNIQUEID_POS);
  u_int uid = Store::WordToInt(*wuid);
#endif
  
  word* wCode = STATIC_CAST(word*, this) - (StackFrame::BASE_SIZE + CODE_POS);
  Chunk* code = Store::DirectWordToChunk(*wCode);

  word* wByteCode = STATIC_CAST(word*, this) - (StackFrame::BASE_SIZE + BYTE_CODE_POS);
  JavaByteCode* bytecode = JavaByteCode::FromWord(*wByteCode);

  word* wPool = STATIC_CAST(word*, this) - (StackFrame::BASE_SIZE + POOL_POS);
  RuntimeConstantPool* pool = RuntimeConstantPool::FromWordDirect(*wPool);

  word* wTop = STATIC_CAST(word*, this) - (StackFrame::BASE_SIZE + TOP_POS);
  u_int top = Store::WordToInt(*wTop);

  word* wContPC = STATIC_CAST(word*, this) - (StackFrame::BASE_SIZE + CONT_PC_POS);
  u_int contPC = Store::WordToInt(*wContPC);

  word* wPC = STATIC_CAST(word*, this) - (StackFrame::BASE_SIZE + PC_POS);
  u_int pc = Store::WordToInt(*wPC);

  word* wSize = STATIC_CAST(word*, this) - (StackFrame::BASE_SIZE + SIZE_POS);
  u_int size = Store::WordToInt(*wSize);

  word* wWorker = STATIC_CAST(word*, this) + StackFrame::WORKER_POS;
  Worker* worker = STATIC_CAST(Worker *, Store::WordToUnmanagedPointer(*wWorker));

  for (int i = bytecode->GetMaxStack() - 1; i >= 0; i--) {
    word* stk = GetBase() - (BASE_SIZE + bytecode->GetMaxLocals() + i);
    fprintf(stderr, "Stack%02d(KaffeFrame):\t0x%08x,\tpos: %p, %02d, %d\n",  i, 
#ifndef DEBUG_NOPOINTERS
            *stk, 
#else
            0,
#endif
            stk, StackFrame::BASE_SIZE + BASE_SIZE + bytecode->GetMaxLocals() + i, BASE_SIZE + bytecode->GetMaxLocals() + i);
  }

  for (int i = bytecode->GetMaxLocals()-1; i >= 0; i--) {
    word* lcl = GetLcl(i);
    fprintf(stderr, "Local%02d(KaffeFrame):\t0x%08x,\tpos: %p, %02d, %d\n", i, 
#ifndef DEBUG_NOPOINTERS
            *lcl, 
#else
            0,
#endif
            lcl, StackFrame::BASE_SIZE + BASE_SIZE + i, BASE_SIZE + i);
  }

#ifdef DEBUG_KAFFEFRAMEID
  fprintf(stderr, "UID(KaffeFrame):\t%010d,\tpos: %p, %02d, %d\n", uid, wuid, StackFrame::BASE_SIZE + UNIQUEID_POS, UNIQUEID_POS);
#endif
  fprintf(stderr, "Code(KaffeFrame):\t%p,\tpos: %p, %02d, %d\n", code, wCode, StackFrame::BASE_SIZE + CODE_POS, CODE_POS);
  fprintf(stderr, "ByteCode(KaffeFrame):\t%p,\tpos: %p, %02d, %d\n", bytecode, wByteCode, StackFrame::BASE_SIZE + BYTE_CODE_POS, BYTE_CODE_POS);
  fprintf(stderr, "Pool(KaffeFrame):\t%p,\tpos: %p, %02d, %d\n", pool, wPool, StackFrame::BASE_SIZE + POOL_POS, POOL_POS);
  fprintf(stderr, "Top(KaffeFrame):\t%010d,\tpos: %p, %02d, %d\n", top, wTop, StackFrame::BASE_SIZE + TOP_POS, TOP_POS);
  fprintf(stderr, "ContPC(KaffeFrame):\t%010d,\tpos: %p, %02d, %d\n", contPC, wContPC, StackFrame::BASE_SIZE + CONT_PC_POS, CONT_PC_POS);
  fprintf(stderr, "PC(KaffeFrame):   \t%010d,\tpos: %p, %02d, %d\n", pc, wPC, StackFrame::BASE_SIZE + PC_POS, PC_POS);
  fprintf(stderr, "Size(KaffeFrame):\t%010d,\tpos: %p, %02d, %d\n", size, wSize, StackFrame::BASE_SIZE + SIZE_POS, SIZE_POS);
  fprintf(stderr, "Worker(KaffeFrame):\t%08p,\tpos: %p\n", worker, wWorker);
}
#endif
