//
// Authors:
//   Patrick Cernko <cernko@ps.uni-sb.de>
//
// Copyright:
//   Patrick Cernko, 2004
//
// Last Change:
//   $Date: 2004/09/06 13:00:57 $ by $Author: errror $
//   $Revision: 1.1.2.2 $
//
#ifndef __JAVA__KAFFE_FRAME_HH__
#define __JAVA__KAFFE_FRAME_HH__

#if defined(INTERFACE)
#pragma interface "java/KaffeFrame.hh"
#endif


#include "Seam.hh"
#include "java/Base.hh"
#include "java/ClassInfo.hh"

class KaffeFrame : public StackFrame {
protected:
public:
  enum {
    SIZE_POS,
    PC_POS,
    CONT_PC_POS,
    TOP_POS,
    POOL_POS,
    BYTE_CODE_POS,
    CODE_POS,
#ifdef DEBUG_KAFFEFRAMEID
    UNIQUEID_POS,
#endif
    BASE_SIZE
  } Positions;
  
  void SetTop(u_int top) {
    StackFrame::ReplaceArg(TOP_POS, top);
  }
  u_int GetTop() {
    return Store::DirectWordToInt(StackFrame::GetArg(TOP_POS));
  }
  word* GetBase() {
    return STATIC_CAST(word*, this) - StackFrame::BASE_SIZE;
  }

  // KaffeFrame Accessors
  u_int GetSize() {
    return Store::DirectWordToInt(StackFrame::GetArg(SIZE_POS));
  }
  s_int GetPC() {
    return Store::DirectWordToInt(StackFrame::GetArg(PC_POS));
  }
  void SetPC(s_int pc) {
    StackFrame::InitArg(PC_POS, pc);
  }
  s_int GetContPC() {
    return Store::DirectWordToInt(StackFrame::GetArg(CONT_PC_POS));
  }
  void SetContPC(s_int pc) {
    StackFrame::InitArg(CONT_PC_POS, pc);
  }
  Chunk *GetCode() {
    return Store::DirectWordToChunk(StackFrame::GetArg(CODE_POS));
  }
  RuntimeConstantPool *GetRuntimeConstantPool() {
    return RuntimeConstantPool::FromWordDirect(StackFrame::GetArg(POOL_POS));
  }
  Table *GetExceptionTable() {
    JavaByteCode *byteCode =
      JavaByteCode::FromWord(StackFrame::GetArg(BYTE_CODE_POS));
    return byteCode->GetExceptionTable();
  }
  MethodInfo *GetMethodInfo() {
    JavaByteCode *byteCode =
      JavaByteCode::FromWord(StackFrame::GetArg(BYTE_CODE_POS));
    return byteCode->GetMethodInfo();
  }

#ifdef DEBUG_KAFFEFRAMEID
  u_int GetUniqueID();
#endif

#define MAX_LOCALS (JavaByteCode::FromWord(StackFrame::GetArg(BYTE_CODE_POS))->GetMaxLocals())
#define STACK_ZERO (BASE_SIZE + MAX_LOCALS)
#define STACK_MAX (STACK_ZERO + JavaByteCode::FromWord(StackFrame::GetArg(BYTE_CODE_POS))->GetMaxStack())

  // Environment Accessors
  word* GetLcl(u_int n) {
    Assert(n < MAX_LOCALS);
    // StackFrames store their members upside down
    return GetBase() - (BASE_SIZE + n);
  }

  // Stack Accessors
  u_int push(u_int n) {
    u_int curTop = GetTop();
#ifdef DEBUG_LEVEL5
    fprintf(stderr, "KaffeFrame::push(%d): new Top: %d\n",n,curTop+n);
#endif
    Assert(curTop + n - 1 <= STACK_MAX);
    SetTop(curTop + n);
    return n;
  }
  u_int pop(u_int n) {
    u_int curTop = GetTop();
#ifdef DEBUG_LEVEL5
    fprintf(stderr, "KaffeFrame::pop(%d): new Top: %d\n",n,curTop-n);
#endif
    Assert(curTop - n >= STACK_ZERO);
    SetTop(curTop - n);
    return n;
  }
  word* GetStack(int n) {
#ifdef DEBUG_LEVEL5
    fprintf(stderr,"KaffeFrame::GetStack(%d)",n);
#endif
    Assert(n <= 0);
    u_int curTop = GetTop();
#ifdef DEBUG_LEVEL5
    fprintf(stderr," Top: %d, this: %p, Base: %p, Pos: %p\n",
            curTop,
            STATIC_CAST(word*, this),
            GetBase(),
            (GetBase() - (curTop + n)));
#endif
    Assert(curTop + n >= STACK_ZERO);
    // StackFrames store their members upside down
    return GetBase() - (curTop + n - 1);
  }
  word GetStackValue(int n) {
    return *GetStack(n);
  }

  static word* GetStack(word* base, u_int top, int n) {
    return base - (top + n - 1);
  }
  static word GetStackValue(word* base, u_int top, int n) {
    return *(base - (top + n - 1));
  }

  // KaffeFrame Constructor
  static KaffeFrame *New(Interpreter *interpreter,
                         s_int pc,
                         s_int contPC,
                         JavaByteCode *byteCode,
                         word runtimeConstantPool);
#ifdef DEBUG_CHECK
  void Dump();
#endif
};

#endif
