//
// Authors:
//   Patrick Cernko <cernko@ps.uni-sb.de>
//
// Copyright:
//   Patrick Cernko, 2004
//
// Last Change:
//   $Date: 2004/03/15 14:31:43 $ by $Author: errror $
//   $Revision: 1.1.2.1 $
//

#if defined(INTERFACE)
#pragma implementation "java/KaffeInterpreterHelper.hh"
#endif

#include "KaffeInterpreterHelpers.hh"

bool IsSubtypeOf(Type *type1, Type *type2) {
  switch (type1->GetLabel()) {
  case JavaLabel::Class:
    return type2->GetLabel() == JavaLabel::Class &&
      STATIC_CAST(Class *, type1)->IsSubtypeOf(STATIC_CAST(Class *, type2));
  case JavaLabel::PrimitiveType:
    switch (type2->GetLabel()) {
    case JavaLabel::Class:
      //--** can be one of the interfaces implemented by arrays (2.15)
      return STATIC_CAST(Class *, type2)->IsJavaLangObject();
    case JavaLabel::PrimitiveType:
      return (STATIC_CAST(PrimitiveType *, type1)->GetType() ==
	      STATIC_CAST(PrimitiveType *, type2)->GetType());
    case JavaLabel::ArrayType:
      return false;
    default:
      Error("invalid type");
    }
  case JavaLabel::ArrayType:
    switch (type2->GetLabel()) {
    case JavaLabel::Class:
      //--** can be one of the interfaces implemented by arrays (2.15)
      return STATIC_CAST(Class *, type2)->IsJavaLangObject();
    case JavaLabel::PrimitiveType:
      return false;
    case JavaLabel::ArrayType:
      {
	Type *elementType1 = STATIC_CAST(ArrayType *, type1)->GetElementType();
	Type *elementType2 = STATIC_CAST(ArrayType *, type2)->GetElementType();
	return IsSubtypeOf(elementType1, elementType2);
      }
    default:
      Error("invalid type");
    }
  default:
    Error("invalid value");
  }
}

bool IsInstanceOf(word value, Type *type) {
  Block *b = Store::WordToBlock(value);
  if (b == INVALID_POINTER) {
    Assert(Store::WordToInt(value) == 0); // null
    return true;
  }
  switch (b->GetLabel()) {
  case JavaLabel::Object:
    return type->GetLabel() == JavaLabel::Class &&
      STATIC_CAST(Object *, b)->IsInstanceOf(STATIC_CAST(Class *, type));
  case JavaLabel::BaseArray:
    switch (type->GetLabel()) {
    case JavaLabel::Class:
      //--** can be one of the interfaces implemented by arrays (2.15)
      return STATIC_CAST(Class *, type)->IsJavaLangObject();
    case JavaLabel::PrimitiveType:
      return false;
    case JavaLabel::ArrayType:
      {
	Type *elementType = STATIC_CAST(ArrayType *, type)->GetElementType();
	return elementType->GetLabel() == JavaLabel::PrimitiveType &&
	  STATIC_CAST(PrimitiveType *, elementType)->GetType() ==
	  STATIC_CAST(BaseArray *, b)->GetElementType();
      }
    default:
      Error("invalid type");
    }
  case JavaLabel::ObjectArray:
    switch (type->GetLabel()) {
    case JavaLabel::Class:
      //--** can be one of the interfaces implemented by arrays (2.15)
      return STATIC_CAST(Class *, type)->IsJavaLangObject();
    case JavaLabel::PrimitiveType:
      return false;
    case JavaLabel::ArrayType:
      {
	Type *elementType1 = STATIC_CAST(ObjectArray *, b)->GetElementType();
	Type *elementType2 = STATIC_CAST(ArrayType *, type)->GetElementType();
	return IsSubtypeOf(elementType1, elementType2);
      }
    default:
      Error("invalid type");
    }
  default:
    Error("invalid value");
  }
}
