//
// Authors:
//   Patrick Cernko <cernko@ps.uni-sb.de>
//
// Copyright:
//   Patrick Cernko, 2004
//
// Last Change:
//   $Date: 2004/03/22 11:28:35 $ by $Author: errror $
//   $Revision: 1.2.2.1 $
//
#ifndef __JAVA__RUNTIME_CONSTANT_POOL_HH__
#define __JAVA__RUNTIME_CONSTANT_POOL_HH__

#if defined(INTERFACE)
#pragma interface "java/RuntimeConstantPool.hh"
#endif

#include "java/Data.hh"
#include "java/ConstantPoolEntry.hh"

//
// Runtime Constant Pool Entries
//
class JavaDll RuntimeConstantPool: private Block {
private:
  u_int GetValueIndex(u_int idx) {
    // the values are located at the even indices
    return (idx - 1) * 2;
  }

  u_int GetCPEIndex(u_int idx) {
    // the CPEs are located at the odd indices
    return (idx - 1) * 2 + 1;
  }

public:
  using Block::ToWord;

  static RuntimeConstantPool *New(u_int size) {
    Block *b = Store::AllocBlock(JavaLabel::RuntimeConstantPool, size*2);
    return STATIC_CAST(RuntimeConstantPool *, b);
  }
  static RuntimeConstantPool *FromWordDirect(word x) {
    Block *b = Store::DirectWordToBlock(x);
    Assert(b->GetLabel() == JavaLabel::RuntimeConstantPool);
    return STATIC_CAST(RuntimeConstantPool *, b);
  }

  void Init(u_int index, word value, ConstantPoolEntry* cpe) {
    InitArg(GetValueIndex(index), value);
    InitArg(GetCPEIndex(index),   cpe->ToWord());
  }
  word Get(u_int index) {
    return GetArg(GetValueIndex(index));
  }
  ConstantPoolEntry* GetTypeInfo(u_int index) {
    return ConstantPoolEntry::FromWordDirect(GetArg(GetCPEIndex(index)));
  }
};

#endif
