//
// Authors:
//   Patrick Cernko <cernko@ps.uni-sb.de>
//
// Copyright:
//   Patrick Cernko, 2004
//
// Last Change:
//   $Date: 2004/09/06 12:50:46 $ by $Author: errror $
//   $Revision: 1.1.2.2 $
//

#if defined(INTERFACE)
#pragma implementation "java/UnlockWorker.hh"
#endif

#include "UnlockWorker.hh"

UnlockWorker *UnlockWorker::self;

class UnlockFrame : public StackFrame {
protected:
  enum {
    LOCK_POS, SIZE
  };
public:
  // UnlockFrame Accessors
  u_int GetSize() {
    return StackFrame::GetSize() + SIZE;
  }
  Lock *GetLock() {
    return Lock::FromWordDirect(StackFrame::GetArg(LOCK_POS));
  }
  // UnlockFrame Constructor
  static UnlockFrame *New(Worker *worker, Lock *lock) {
#ifdef DEBUG_LEVEL4
  fprintf(stderr, "UnlockFrame::New(%p, %p) == ",
          worker,
          lock);
#endif
    NEW_STACK_FRAME(frame, worker, SIZE);
#ifdef DEBUG_LEVEL4
    fprintf(stderr, "%p\n", frame);
#endif
    frame->InitArg(LOCK_POS, lock->ToWord());
    return STATIC_CAST(UnlockFrame *, frame);
  }
};

void UnlockWorker::PushFrame(Lock *lock) {
  UnlockFrame::New(UnlockWorker::self, lock);
  Scheduler::PushHandler(Store::IntToWord(0));
}

void UnlockWorker::Release(StackFrame *sFrame) {
  UnlockFrame *frame = STATIC_CAST(UnlockFrame *, sFrame);
  Assert(sFrame->GetWorker() == UnlockWorker::self);
  Scheduler::PopHandler();
  Lock *lock = frame->GetLock();
  Scheduler::PopFrame(frame->GetSize());
  lock->Release();
}

u_int UnlockWorker::GetFrameSize(StackFrame *sFrame) {
  UnlockFrame *frame = STATIC_CAST(UnlockFrame *, sFrame);
  Assert(sFrame->GetWorker() == this);
  return frame->GetSize();
}

Worker::Result UnlockWorker::Run(StackFrame *sFrame) {
  Release(sFrame);
  return Worker::CONTINUE;
}

Worker::Result UnlockWorker::Handle(word) {
  Release(Scheduler::GetFrame());
  return Worker::RAISE;
}

const char *UnlockWorker::Identify() {
  return "UnlockWorker";
}

void UnlockWorker::DumpFrame(StackFrame *) {
  std::fprintf(stderr, "Unlock\n");
}
