/*******************************************************/
/* Authors:                                            */
/*   Patrick Cernko <cernko@ps.uni-sb.de>              */
/*                                                     */
/* Copyright:                                          */
/*   Patrick Cernko, 2004                              */
/*                                                     */
/* Last Change:                                        */
/*   $Date: 2004/09/06 13:00:57 $ by $Author: errror $ */
/*   $Revision: 1.1.2.1 $                              */
/*******************************************************/


#include "fp.h"

/*
 * Convert float to int.
 */
jint
floatToInt(jfloat val)
{
        jvalue d;

	d.f = val;
	return d.i;
}

/*
 * Convert double to jlong.
 *
 * If you change this function, also change strtod("-0.0") check in
 * configure.in.
 */
jlong
doubleToLong(jdouble val)
{
	jvalue d;
	d.d = val;

#if defined(DOUBLE_ORDER_OPPOSITE)
	{
		/* swap low and high word */
		uint32 r = *(uint32*)&d.j;
		uint32 *s = (uint32*)&d.j + 1;
		d.i = *s;
		*s = r;
	}
#endif
	return d.j;
}

jlong
soft_cvtfl(jfloat v)
{
	jint vbits;

	vbits = floatToInt(v);
        if (FISNAN(vbits)) {
		return ((jlong)0);
	}

	if (v < 0.0) {
		v = ceil(v);
	}
	else {
		v = floor(v);
	}
	/* If too small return smallest long */
	if (v <= -9223372036854775808.0) {
	    return ((jlong)1) << 63;
	}
	/* If too big return biggest long */
	else if (v >= 9223372036854775807.0) {
	    return ~(((jlong)1) << 63);
	}
	else {
	    return ((jlong)v);
	}
}

jint
soft_fcmpg(jfloat v1, jfloat v2)
{
        jint ret;
	jint v1bits;
	jint v2bits;

	v1bits = floatToInt(v1);
	v2bits = floatToInt(v2);

        if (FISNAN(v1bits) || FISNAN(v2bits)) {
		ret = 1;
	}
        else if (v1 > v2) {
                ret = 1;
        }
        else if (v1 == v2) {
                ret = 0;
        }
        else {
                ret = -1;
        }
	return (ret);
}

void*
soft_anewarray(Hjava_lang_Class* /*elclass*/, jint /*size*/)
{
  dprintf("NOT IMPLEMENTED: %s:%d %s()\n",__FILE__,__LINE__,__FUNCTION__); exit(1);
}

void*
soft_checkcast(Hjava_lang_Class* /*c*/, Hjava_lang_Object* /*o*/)
{
  dprintf("NOT IMPLEMENTED: %s:%d %s()\n",__FILE__,__LINE__,__FUNCTION__); exit(1);
}

void
soft_illegalaccess(Utf8Const */*c*/, Utf8Const* /*n*/)
{
  dprintf("NOT IMPLEMENTED: %s:%d %s()\n",__FILE__,__LINE__,__FUNCTION__); exit(1);
}

void*
soft_new(Hjava_lang_Class* /*c*/)
{
  dprintf("NOT IMPLEMENTED: %s:%d %s()\n",__FILE__,__LINE__,__FUNCTION__); exit(1);
}

jint
soft_fcmpl(jfloat v1, jfloat v2)
{
        jint ret;
	jint v1bits;
	jint v2bits;

	v1bits = floatToInt(v1);
	v2bits = floatToInt(v2);

        if (FISNAN(v1bits) || FISNAN(v2bits)) {
		ret = -1;
	}
        else if (v1 > v2) {
                ret = 1;
        }
        else if (v1 == v2) {
                ret = 0;
        }
        else {
                ret = -1;
        }
	return (ret);
}

jint
soft_dcmpl(jdouble v1, jdouble v2)
{
        jint ret;
	if ((!isinf(v1) && isnan(v1)) || (!isinf(v2) && isnan(v2))) {
		ret = -1;
	}
        else if (v1 > v2) {
                ret = 1;
        }
        else if (v1 == v2) {
                ret = 0;
        }
        else {
                ret = -1;
        }
	return (ret);
}

jlong
soft_cvtdl(jdouble v)
{
	jlong vbits;

	vbits = doubleToLong(v);
        if (DISNAN(vbits)) {
		return ((jlong)0);
	}

	if (v < 0.0) {
		v = ceil(v);
	}
	else {
		v = floor(v);
	}
	/* If too small return smallest long */
	if (v <= -9223372036854775808.0) {
	    return ((jlong)1) << 63;
	}
	/* If too big return biggest long */
	else if (v >= 9223372036854775807.0) {
	    return ~(((jlong)1) << 63);
	}
	else {
	    return ((jlong)v);
	}
}

void*
soft_newarray(jint /*type*/, jint /*size*/)
{
  dprintf("NOT IMPLEMENTED: %s:%d %s()\n",__FILE__,__LINE__,__FUNCTION__); exit(1);
}

void
soft_incompatibleclasschange(Utf8Const */*c*/, Utf8Const* /*n*/)
{
  dprintf("NOT IMPLEMENTED: %s:%d %s()\n",__FILE__,__LINE__,__FUNCTION__); exit(1);
}

void
soft_checkarraystore(Hjava_lang_Object* /*array*/, Hjava_lang_Object* /*obj*/)
{
  dprintf("NOT IMPLEMENTED: %s:%d %s()\n",__FILE__,__LINE__,__FUNCTION__); exit(1);
}

jint
soft_dcmpg(jdouble /*v1*/, jdouble /*v2*/)
{
  dprintf("NOT IMPLEMENTED: %s:%d %s()\n",__FILE__,__LINE__,__FUNCTION__); exit(1);
}

jint
soft_instanceof(Hjava_lang_Class* /*c*/, Hjava_lang_Object* /*o*/)
{
  dprintf("NOT IMPLEMENTED: %s:%d %s()\n",__FILE__,__LINE__,__FUNCTION__); exit(1);
}

void
soft_nullpointer(void)
{
  dprintf("NOT IMPLEMENTED: %s:%d %s()\n",__FILE__,__LINE__,__FUNCTION__); exit(1);
}

void
soft_nosuchfield(Utf8Const* /*c*/, Utf8Const* /*n*/)
{
  dprintf("NOT IMPLEMENTED: %s:%d %s()\n",__FILE__,__LINE__,__FUNCTION__); exit(1);
}

void*
soft_multianewarray(Hjava_lang_Class* /*cls*/, jint /*dims*/, slots* /*args*/)
{
  dprintf("NOT IMPLEMENTED: %s:%d %s()\n",__FILE__,__LINE__,__FUNCTION__); exit(1);
  return NULL;
}

void
soft_badarrayindex(void)
{
  dprintf("NOT IMPLEMENTED: %s:%d %s()\n",__FILE__,__LINE__,__FUNCTION__); exit(1);
}

jint
soft_cvtfi(jfloat v)
{
        jint vbits;

	vbits = floatToInt(v);
        if (FISNAN(vbits)) {
		return (0);
	}

	if (v < 0.0) {
		v = ceil(v);
	}
	else {
		v = floor(v);
	}
	/* If too small return smallest int */
	if (v <= -2147483648.0) {
		return (-2147483647-1);
	}
	/* If too big return biggest int */
	else if (v >= 2147483647.0) {
		return (2147483647);
	}
	else {
		return (jint)v;
	}
}

void
soft_initialise_class(Hjava_lang_Class* /*c*/)
{
  dprintf("NOT IMPLEMENTED: %s:%d %s()\n",__FILE__,__LINE__,__FUNCTION__); exit(1);
}

void
soft_null_call(void)
{
  dprintf("NOT IMPLEMENTED: %s:%d %s()\n",__FILE__,__LINE__,__FUNCTION__); exit(1);
}

jint
soft_cvtdi(jdouble v)
{
        jlong vbits;

	vbits = doubleToLong(v);
        if (DISNAN(vbits)) {
		return (0);
	}

	if (v < 0.0) {
		v = ceil(v);
	}
	else {
		v = floor(v);
	}
	/* If too small return smallest int */
	if (v <= -2147483648.0) {
		return (-2147483647-1);
	}
	/* If too big return biggest int */
	else if (v >= 2147483647.0) {
		return (2147483647);
	}
	else {
		return ((jint)v);
	}
}

void
soft_athrow(Hjava_lang_Object* /*o*/)
{
  dprintf("NOT IMPLEMENTED: %s:%d %s()\n",__FILE__,__LINE__,__FUNCTION__); exit(1);
}

void
soft_nosuchmethod(Hjava_lang_Class* /*c*/, Utf8Const* /*n*/, Utf8Const* /*s*/)
{
  dprintf("NOT IMPLEMENTED: %s:%d %s()\n",__FILE__,__LINE__,__FUNCTION__); exit(1);
}

const uint8 insnLen[256] = {
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
	2, 3, 2, 3, 3, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 
	3, 3, 3, 3, 3, 3, 3, 3, 3, 2, 0, 0, 1, 1, 1, 1, 
	1, 1, 3, 3, 3, 3, 3, 3, 3, 5, 1, 3, 2, 3, 1, 1, 
	3, 3, 1, 1, 1, 4, 3, 3, 5, 5, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
};

